@@ -5,8 +5,10 @@
 # @License  : GNU General Public License v3.0
 # @Author   : xxx
 import os
+
 from mgetool.imports import BatchFileMatch
-from mgetool.imports.batchfilematch import re_patten_help, shell_patten_help
+from mgetool.imports.batchfilematch import re_patten_help, shell_patten_help, BatchPathMatch
+
 
 _dos_help = f"""寻找符合要求所有的叶节点路径, 查看参数帮助使用 -h.
 
@@ -58,36 +60,52 @@ Key 4. 多重可选匹配使用 | 或者空格划分.
 
 def run(args, parser):
     print("Collecting all Paths ...")
+    if args.suffix is None and\
+       args.dir_include is None and args.dir_exclude is None and args.file_include is None and\
+       args.file_exclude is None:
+
+        if args.match_patten_arg is None:
+            bf = BatchPathMatch(args.path, patten=args.match_patten, trans=args.translate,
+                                abspath=args.abspath)
+        else:
+            bf = BatchPathMatch(args.path, patten=args.match_patten_arg, trans=args.translate,
+                                abspath=args.abspath)
 
-    # situation 1
-    # if the 'match_patten_arg' is use [^...] or [!seq] just for match file name,
-    # This would find all file matched with patten in the dirs,
-    # The dirs would remain due to the file. thus the  [^...] or [!seq] (for file name) would not filter the dirs.
-    if args.match_patten_arg is None:
-        bf = BatchFileMatch(args.path, suffix=args.suffix, patten=args.match_patten, trans=args.translate)
     else:
-        bf = BatchFileMatch(args.path, suffix=args.suffix, patten=args.match_patten_arg, trans=args.translate)
 
-    print("Filter the Paths ...")
+        # situation 1
+        # if the 'match_patten_arg' is use [^...] or [!seq] just for match file name,
+        # This would find all file matched with patten in the dirs,
+        # The dirs would remain due to the file. thus the  [^...] or [!seq] (for file name) would not filter the dirs.
+        if args.match_patten_arg is None:
+            bf = BatchFileMatch(args.path, suffix=args.suffix, patten=args.match_patten, trans=args.translate)
+        else:
+            bf = BatchFileMatch(args.path, suffix=args.suffix, patten=args.match_patten_arg, trans=args.translate)
 
-    # situation 2
-    # (the parent of parent dir or more top-lever could be residual. if dir_exclude not None)
-    bf.filter_dir_name(include=args.dir_include, exclude=args.dir_exclude, layer=args.layer)
+        print("Filter the Paths ...")
 
-    # situation 1
-    # if use 'exclude' in this function, the dirs containing exclude file would remain, due to the other file in dirs.
-    # thus, exclude are set to next function.
-    bf.filter_file_name(include=args.file_include)
+        # situation 2
+        # (the parent of parent dir or more top-lever could be residual. if dir_exclude not None)
+        bf.filter_dir_name(include=args.dir_include, exclude=args.dir_exclude, layer=args.layer)
 
-    # this is the real, to delete the dirs containing exclude file.
-    bf.filter_file_name_parent_folder(exclude=args.file_exclude)
+        # situation 1
+        # if use 'exclude' in this function, the dirs containing exclude file would remain, due to the other file in dirs.
+        # thus, exclude are set to next function.
+        bf.filter_file_name(include=args.file_include)
 
-    if args.dir_exclude is not None:
-        print("Use '-ed' could result to parent folder residue. Manual check and delete is recommended.")
+        # this is the real, to delete the dirs containing exclude file.
+        bf.filter_file_name_parent_folder(exclude=args.file_exclude)
 
-    bf.merge(abspath=args.abspath)
+        if args.dir_exclude is not None:
+            print("Use '-ed' could result to parent folder residue. Manual check and delete is recommended.")
+
+        bf.merge(abspath=args.abspath)
+
+    if not args.parent:
+        fdir = bf.get_leaf_dir()
+    else:
+        fdir = bf.file_dir
 
-    fdir = bf.get_leaf_dir()
     num = len(fdir)
 
     if args.not_print is True:
@@ -101,7 +119,7 @@ def run(args, parser):
         fdir = "\n".join(fdir)
         f.writelines(fdir)
 
-    print("The '{}' of {} paths are stored in '{}'.".format(args.store_name, num,  str(os.getcwd())))
+    print("The '{}' of {} paths are stored in '{}'.".format(args.store_name, num, str(os.getcwd())))
     print("OK")
 
 
@@ -125,6 +143,7 @@ class CLICommand:
         parser.add_argument('-o', '--store_name', help='out file name, default paths.temp.', type=str,
                             default="paths.temp")
         parser.add_argument('-np', '--not_print', help='not print.', action="store_false")
+        parser.add_argument('-parent', '--parent', help='with parent or not.', action="store_true")
 
     @staticmethod
     def parse_args(parser):
