@@ -44,11 +44,14 @@ a|b     匹配a或b.
 """
 
 
-def shell_to_re_compile_pattern(pat, trans=True):
+def shell_to_re_compile_pattern(pat, trans=True, single=True, ):
     """shell to re patten."""
     try:
         if trans:
             res = translate(pat)
+            if single is False:
+                res = res.replace("?s:", "?m:")
+                res = res.replace("\Z", "")
         else:
             res = pat
         return re.compile(res)
@@ -67,6 +70,84 @@ def shell_to_re_compile_pattern(pat, trans=True):
         raise e
 
 
+class BatchPathMatch:
+    def __init__(self, path=".", patten: Union[str]=None, trans=True, abspath=False):
+        """
+
+        Parameters
+        ----------
+        path:str
+            total dir of all files.
+        patten:str
+            match patten.
+        trans: bool, default True
+            If true, use shell patten to match.
+            If False, use re patten to match.
+        """
+        self.trans = trans
+
+        self.path = Path(path)
+        if patten is not None:
+
+            dir_list = self.path.walkdirs()
+            patten = self._get_patten(patten)
+            self.file_dir = list(self.filter(dir_list, pat=patten, trans=self.trans))
+
+        else:
+            self.file_dir = list(self.path.walkdirs())
+        if abspath:
+            self.file_dir  = self.dir_relpath()
+
+    @staticmethod
+    def _get_patten(my_strs):
+        if my_strs is not None and " " in my_strs:
+            my_strs = str(my_strs).replace("|", " ")
+            my_strs = my_strs.split(" ")
+            my_strs = [i for i in my_strs if i not in ["", " "]]
+        else:
+            pass
+
+        if isinstance(my_strs, (tuple, list)):
+            if len(my_strs) >= 2:
+                my_strs = "|".join(my_strs)
+            elif len(my_strs) == 1:
+                my_strs = my_strs[0]
+            else:
+                raise NotImplementedError
+        else:
+            pass
+        return my_strs
+
+    @staticmethod
+    def filter(names, pat, trans=True):
+        smatch = shell_to_re_compile_pattern(pat, trans=trans).search
+        return [ni for ni in names if smatch(ni) is not None]
+
+    def get_leaf_dir(self, dirs=None):
+        """Get the leaf dirs. Remove "." path."""
+
+        # The ../paths could be wrong!
+
+        if dirs is None:
+            dirs = self.file_dir
+
+        dirs = [i for i in dirs if i not in (".", "./")]
+        dirs_text = "\n".join(dirs)
+        index = [dirs_text.count(i) - 1 for i in dirs]
+        dirs = [dirs[n] for n, i in enumerate(index) if i == 0]
+        return dirs
+
+    def dir_relpath(self, path=None):
+        """Get the real-path to input path."""
+        if path is None:
+            path = self.path
+        else:
+            path = Path(path)
+
+        self.file_dir = [i.relpath(path) for i in self.file_dir]
+
+
+
 class BatchFileMatch:
     r"""
     Search files and filter files and re-site files with patten match.
@@ -105,7 +186,7 @@ class BatchFileMatch:
             If true, use shell patten to match.
             If False, use re patten to match.
         """
-        self.trans=trans
+        self.trans = trans
 
         self.path = Path(path)
         if patten is not None:
@@ -169,6 +250,7 @@ class BatchFileMatch:
             pass
         return my_strs
 
+
     def filter_file_name_parent_folder(self, exclude=None):
         """
         filter the dir and sub-files contain the file!
@@ -232,7 +314,7 @@ class BatchFileMatch:
 
                 if include:
                     include = self._get_patten(include)
-                    if not any([i in include for i in ["*","|","?",".","\\","[","+"]]):# 简单模式
+                    if not any([i in include for i in ["*", "|", "?", ".", "\\", "[", "+"]]):  # 简单模式
                         self.file_list = [self.file_list[r] for r, i in enumerate(files) if include in i]
                         files = [i for i in files if include in i] if exclude else []
                     else:
@@ -243,7 +325,7 @@ class BatchFileMatch:
 
                 if exclude:
                     exclude = self._get_patten(exclude)
-                    if not any([i in exclude for i in ["*","|","?",".","\\","[","+"]]):# 简单模式
+                    if not any([i in exclude for i in ["*", "|", "?", ".", "\\", "[", "+"]]):  # 简单模式
                         self.file_list = [self.file_list[r] for r, i in enumerate(files) if exclude not in i]
                     else:
                         pt2 = self._comp(exclude)
@@ -314,7 +396,7 @@ class BatchFileMatch:
                 # 只要出现include内容 - 保留,或者只要出现exclude内容 - 删除
                 if include:
                     include = self._get_patten(include)
-                    if not any([i in include for i in ["*","|","?",".","\\","[","+"]]):  # 简单模式
+                    if not any([i in include for i in ["*", "|", "?", ".", "\\", "[", "+"]]):  # 简单模式
                         self.file_list = [self.file_list[r] for r, i in enumerate(file_dir) if include in i]
                         file_dir = [i for i in file_dir if include in i] if exclude else []
                     else:
@@ -325,14 +407,16 @@ class BatchFileMatch:
 
                 if exclude:
                     exclude = self._get_patten(exclude)
-                    if not any([i in exclude for i in ["*","|","?",".","\\","[","+"]]):# 简单模式
+                    if not any([i in exclude for i in ["*", "|", "?", ".", "\\", "[", "+"]]):  # 简单模式
                         self.file_list = [self.file_list[r] for r, i in enumerate(file_dir) if exclude not in i]
                     else:
                         pt2 = self._comp(exclude)
-                        self.file_list = [self.file_list[r] for r, i in enumerate(file_dir) if re.search(pt2, i) is None]
+                        self.file_list = [self.file_list[r] for r, i in enumerate(file_dir) if
+                                          re.search(pt2, i) is None]
 
     def merge(self, abspath=False, force_relpath=False):
         """Merge dir and file name together, Get dir names."""
+
         if abspath:
             self.file_list = [i.abspath() for i in self.file_list]
         else:
@@ -456,7 +540,6 @@ class BatchFileMatch:
         dirs = [dirs[n] for n, i in enumerate(index) if i == 0]
         return dirs
 
-
 # if __name__=="__main__":
 #     bfm = BatchFileMatch(r"C:\Users\Administrator\PycharmProjects\samples\Instance\Instance_mo2co2\MoCMo-O-4")
 #
